@@ -30,12 +30,12 @@
  **/
 
 
-#include "DicomIntegerPixelAccessor.h"
-
 #ifndef NOMINMAX
 #define NOMINMAX
 #endif
 
+#include "DicomIntegerPixelAccessor.h"
+
 #include "../OrthancException.h"
 #include <boost/lexical_cast.hpp>
 #include <limits>
@@ -140,11 +140,13 @@ namespace Orthanc
 
     if (pixelRepresentation)
     {
+      // Pixels are signed
       mask_ = (1 << (bitsStored - 1)) - 1;
       signMask_ = (1 << (bitsStored - 1));
     }
     else
     {
+      // Pixels are unsigned
       mask_ = (1 << bitsStored) - 1;
       signMask_ = 0;
     }
@@ -231,25 +233,28 @@ namespace Orthanc
       pixel += channel * frameOffset_ / samplesPerPixel_ + x * bytesPerPixel_;
     }
 
-    int32_t v;
+    uint32_t v;
     v = pixel[0];
     if (bytesPerPixel_ >= 2)
-      v = v + (static_cast<int32_t>(pixel[1]) << 8);
+      v = v + (static_cast<uint32_t>(pixel[1]) << 8);
     if (bytesPerPixel_ >= 3)
-      v = v + (static_cast<int32_t>(pixel[2]) << 16);
+      v = v + (static_cast<uint32_t>(pixel[2]) << 16);
     if (bytesPerPixel_ >= 4)
-      v = v + (static_cast<int32_t>(pixel[3]) << 24);
+      v = v + (static_cast<uint32_t>(pixel[3]) << 24);
 
-    v = (v >> shift_) & mask_;
+    v = v >> shift_;
 
     if (v & signMask_)
     {
-      // Signed value: Not implemented yet
-      //throw OrthancException(ErrorCode_NotImplemented);
-      v = 0;
+      // Signed value
+      // http://en.wikipedia.org/wiki/Two%27s_complement#Subtraction_from_2N
+      return -static_cast<int32_t>(mask_) + static_cast<int32_t>(v & mask_) - 1;
+    }
+    else
+    {
+      // Unsigned value
+      return static_cast<int32_t>(v & mask_);
     }
-
-    return v;
   }
 
 
