@@ -56,7 +56,7 @@ namespace Orthanc
 
     virtual void Handle(
       HttpOutput& output,
-      Orthanc_HttpMethod method,
+      HttpMethod method,
       const UriComponents& uri,
       const Arguments& headers,
       const Arguments& arguments,
