@@ -36,7 +36,6 @@
 #include <vector>
 #include <stdint.h>
 #include "../Toolbox.h"
-#include "../../OrthancCppClient/HttpEnumerations.h"
 
 namespace Orthanc
 {
@@ -54,7 +53,7 @@ namespace Orthanc
     virtual bool IsServedUri(const UriComponents& uri) = 0;
 
     virtual void Handle(HttpOutput& output,
-                        Orthanc_HttpMethod method,
+                        HttpMethod method,
                         const UriComponents& uri,
                         const Arguments& headers,
                         const Arguments& getArguments,
