@@ -38,7 +38,6 @@
 #include <boost/lexical_cast.hpp>
 #include "../OrthancException.h"
 #include "../Toolbox.h"
-#include "../../OrthancCppClient/HttpException.h"
 
 namespace Orthanc
 {
@@ -90,7 +89,7 @@ namespace Orthanc
     std::string s = "HTTP/1.1 200 OK\r\n";
 
     for (Header::const_iterator 
-           it = header.begin(); it != header.end(); it++)
+           it = header.begin(); it != header.end(); ++it)
     {
       s += it->first + ": " + it->second + "\r\n";
     }
@@ -104,17 +103,17 @@ namespace Orthanc
   void HttpOutput::SendMethodNotAllowedError(const std::string& allowed)
   {
     std::string s = 
-      "HTTP/1.1 405 " + std::string(HttpException::GetDescription(Orthanc_HttpStatus_405_MethodNotAllowed)) +
+      "HTTP/1.1 405 " + std::string(EnumerationToString(HttpStatus_405_MethodNotAllowed)) +
       "\r\nAllow: " + allowed + 
       "\r\n\r\n";
     Send(&s[0], s.size());
   }
 
 
-  void HttpOutput::SendHeader(Orthanc_HttpStatus status)
+  void HttpOutput::SendHeader(HttpStatus status)
   {
-    if (status == Orthanc_HttpStatus_200_Ok ||
-        status == Orthanc_HttpStatus_405_MethodNotAllowed)
+    if (status == HttpStatus_200_Ok ||
+        status == HttpStatus_405_MethodNotAllowed)
     {
       throw OrthancException("Please use the dedicated methods to this HTTP status code in HttpOutput");
     }
@@ -123,11 +122,11 @@ namespace Orthanc
   }
 
 
-  void HttpOutput::SendHeaderInternal(Orthanc_HttpStatus status)
+  void HttpOutput::SendHeaderInternal(HttpStatus status)
   {
     std::string s = "HTTP/1.1 " + 
       boost::lexical_cast<std::string>(status) +
-      " " + std::string(HttpException::GetDescription(status)) +
+      " " + std::string(EnumerationToString(status)) +
       "\r\n\r\n";
     Send(&s[0], s.size());
   }
@@ -145,7 +144,7 @@ namespace Orthanc
                                   const HttpHandler::Arguments& cookies)
   {
     for (HttpHandler::Arguments::const_iterator it = cookies.begin();
-         it != cookies.end(); it++)
+         it != cookies.end(); ++it)
     {
       header.push_back(std::make_pair("Set-Cookie", it->first + "=" + it->second));
     }
@@ -190,7 +189,7 @@ namespace Orthanc
   void HttpOutput::Redirect(const std::string& path)
   {
     std::string s = 
-      "HTTP/1.1 301 " + std::string(HttpException::GetDescription(Orthanc_HttpStatus_301_MovedPermanently)) + 
+      "HTTP/1.1 301 " + std::string(EnumerationToString(HttpStatus_301_MovedPermanently)) + 
       "\r\nLocation: " + path +
       "\r\n\r\n";
     Send(&s[0], s.size());  
