@@ -51,7 +51,7 @@ namespace Orthanc
     result.clear();
 
     for (HttpHandler::Arguments::const_iterator 
-           it = getArguments_->begin(); it != getArguments_->end(); it++)
+           it = getArguments_.begin(); it != getArguments_.end(); ++it)
     {
       result[it->first] = it->second;
     }
@@ -63,7 +63,7 @@ namespace Orthanc
   bool RestApi::IsGetAccepted(const UriComponents& uri)
   {
     for (GetHandlers::const_iterator it = getHandlers_.begin();
-         it != getHandlers_.end(); it++)
+         it != getHandlers_.end(); ++it)
     {
       if (it->first->Match(uri))
       {
@@ -77,7 +77,7 @@ namespace Orthanc
   bool RestApi::IsPutAccepted(const UriComponents& uri)
   {
     for (PutHandlers::const_iterator it = putHandlers_.begin();
-         it != putHandlers_.end(); it++)
+         it != putHandlers_.end(); ++it)
     {
       if (it->first->Match(uri))
       {
@@ -91,7 +91,7 @@ namespace Orthanc
   bool RestApi::IsPostAccepted(const UriComponents& uri)
   {
     for (PostHandlers::const_iterator it = postHandlers_.begin();
-         it != postHandlers_.end(); it++)
+         it != postHandlers_.end(); ++it)
     {
       if (it->first->Match(uri))
       {
@@ -105,7 +105,7 @@ namespace Orthanc
   bool RestApi::IsDeleteAccepted(const UriComponents& uri)
   {
     for (DeleteHandlers::const_iterator it = deleteHandlers_.begin();
-         it != deleteHandlers_.end(); it++)
+         it != deleteHandlers_.end(); ++it)
     {
       if (it->first->Match(uri))
       {
@@ -147,25 +147,25 @@ namespace Orthanc
   RestApi::~RestApi()
   {
     for (GetHandlers::iterator it = getHandlers_.begin(); 
-         it != getHandlers_.end(); it++)
+         it != getHandlers_.end(); ++it)
     {
       delete it->first;
     } 
 
     for (PutHandlers::iterator it = putHandlers_.begin(); 
-         it != putHandlers_.end(); it++)
+         it != putHandlers_.end(); ++it)
     {
       delete it->first;
     } 
 
     for (PostHandlers::iterator it = postHandlers_.begin(); 
-         it != postHandlers_.end(); it++)
+         it != postHandlers_.end(); ++it)
     {
       delete it->first;
     } 
 
     for (DeleteHandlers::iterator it = deleteHandlers_.begin(); 
-         it != deleteHandlers_.end(); it++)
+         it != deleteHandlers_.end(); ++it)
     {
       delete it->first;
     } 
@@ -180,7 +180,7 @@ namespace Orthanc
   }
 
   void RestApi::Handle(HttpOutput& output,
-                       Orthanc_HttpMethod method,
+                       HttpMethod method,
                        const UriComponents& uri,
                        const Arguments& headers,
                        const Arguments& getArguments,
@@ -191,88 +191,58 @@ namespace Orthanc
     RestApiPath::Components components;
     UriComponents trailing;
 
-    if (method == Orthanc_HttpMethod_Get)
+    if (method == HttpMethod_Get)
     {
       for (GetHandlers::const_iterator it = getHandlers_.begin();
-           it != getHandlers_.end(); it++)
+           it != getHandlers_.end(); ++it)
       {
         if (it->first->Match(components, trailing, uri))
         {
           //LOG(INFO) << "REST GET call on: " << Toolbox::FlattenUri(uri);
           ok = true;
-          GetCall call;
-          call.output_ = &restOutput;
-          call.context_ = this;
-          call.httpHeaders_ = &headers;
-          call.uriComponents_ = &components;
-          call.trailing_ = &trailing;
-          call.fullUri_ = &uri;
-          
-          call.getArguments_ = &getArguments;
+          GetCall call(restOutput, *this, headers, components, trailing, uri, getArguments);
           it->second(call);
         }
       }
     }
-    else if (method == Orthanc_HttpMethod_Put)
+    else if (method == HttpMethod_Put)
     {
       for (PutHandlers::const_iterator it = putHandlers_.begin();
-           it != putHandlers_.end(); it++)
+           it != putHandlers_.end(); ++it)
       {
         if (it->first->Match(components, trailing, uri))
         {
           //LOG(INFO) << "REST PUT call on: " << Toolbox::FlattenUri(uri);
           ok = true;
-          PutCall call;
-          call.output_ = &restOutput;
-          call.context_ = this;
-          call.httpHeaders_ = &headers;
-          call.uriComponents_ = &components;
-          call.trailing_ = &trailing;
-          call.fullUri_ = &uri;
-           
-          call.data_ = &postData;
+          PutCall call(restOutput, *this, headers, components, trailing, uri, postData);
           it->second(call);
         }
       }
     }
-    else if (method == Orthanc_HttpMethod_Post)
+    else if (method == HttpMethod_Post)
     {
       for (PostHandlers::const_iterator it = postHandlers_.begin();
-           it != postHandlers_.end(); it++)
+           it != postHandlers_.end(); ++it)
       {
         if (it->first->Match(components, trailing, uri))
         {
           //LOG(INFO) << "REST POST call on: " << Toolbox::FlattenUri(uri);
           ok = true;
-          PostCall call;
-          call.output_ = &restOutput;
-          call.context_ = this;
-          call.httpHeaders_ = &headers;
-          call.uriComponents_ = &components;
-          call.trailing_ = &trailing;
-          call.fullUri_ = &uri;
-           
-          call.data_ = &postData;
+          PostCall call(restOutput, *this, headers, components, trailing, uri, postData);
           it->second(call);
         }
       }
     }
-    else if (method == Orthanc_HttpMethod_Delete)
+    else if (method == HttpMethod_Delete)
     {
       for (DeleteHandlers::const_iterator it = deleteHandlers_.begin();
-           it != deleteHandlers_.end(); it++)
+           it != deleteHandlers_.end(); ++it)
       {
         if (it->first->Match(components, trailing, uri))
         {
           //LOG(INFO) << "REST DELETE call on: " << Toolbox::FlattenUri(uri);
           ok = true;
-          DeleteCall call;
-          call.output_ = &restOutput;
-          call.context_ = this;
-          call.httpHeaders_ = &headers;
-          call.uriComponents_ = &components;
-          call.trailing_ = &trailing;
-          call.fullUri_ = &uri;
+          DeleteCall call(restOutput, *this, headers, components, trailing, uri);
           it->second(call);
         }
       }
@@ -280,7 +250,7 @@ namespace Orthanc
 
     if (!ok)
     {
-      LOG(INFO) << "REST method " << Orthanc_HttpMethod_ToString(method) 
+      LOG(INFO) << "REST method " << EnumerationToString(method) 
                 << " not allowed on: " << Toolbox::FlattenUri(uri);
       output.SendMethodNotAllowedError(GetAcceptedMethods(uri));
     }
