@@ -32,6 +32,8 @@
 
 #pragma once
 
+#include "Enumerations.h"
+
 #include <stdint.h>
 #include <vector>
 #include <string>
@@ -55,6 +57,9 @@ namespace Orthanc
     void ReadFile(std::string& content,
                   const std::string& path);
 
+    void WriteFile(const std::string& content,
+                   const std::string& path);
+
     void Sleep(uint32_t seconds);
 
     void USleep(uint64_t microSeconds);
@@ -101,5 +106,13 @@ namespace Orthanc
 
     // In-place percent-decoding for URL
     void UrlDecode(std::string& s);
+
+    Endianness DetectEndianness();
+
+    std::string WildcardToRegularExpression(const std::string& s);
+
+    void TokenizeString(std::vector<std::string>& result,
+                        const std::string& source,
+                        char separator);
   }
 }
