@@ -108,10 +108,16 @@ namespace Orthanc
                      MetadataType type,
                      const std::string& value);
 
+    void DeleteMetadata(int64_t id,
+                        MetadataType type);
+
     bool LookupMetadata(std::string& target,
                         int64_t id,
                         MetadataType type);
 
+    bool ListAvailableMetadata(std::list<MetadataType>& target,
+                               int64_t id);
+
     std::string GetMetadata(int64_t id,
                             MetadataType type,
                             const std::string& defaultValue = "");
@@ -123,6 +129,9 @@ namespace Orthanc
     void AddAttachment(int64_t id,
                        const FileInfo& attachment);
 
+    void ListAvailableAttachments(std::list<FileContentType>& result,
+                                  int64_t id);
+
     bool LookupAttachment(FileInfo& attachment,
                           int64_t id,
                           FileContentType contentType);
@@ -214,5 +223,14 @@ namespace Orthanc
     uint64_t IncrementGlobalSequence(GlobalProperty property);
 
     void ClearTable(const std::string& tableName);
+
+    bool IsExistingResource(int64_t internalId);
+
+    void LookupTagValue(std::list<int64_t>& result,
+                        DicomTag tag,
+                        const std::string& value);
+
+    void LookupTagValue(std::list<int64_t>& result,
+                        const std::string& value);
   };
 }
