@@ -47,7 +47,8 @@ namespace Orthanc
     {
     }
 
-    virtual void Handle(const DicomMap& input,
-                        DicomFindAnswers& answers) = 0;
+    virtual void Handle(DicomFindAnswers& answers,
+                        const DicomMap& input,
+                        const std::string& callingAETitle) = 0;
   };
 }
