@@ -43,13 +43,6 @@
 
 namespace Orthanc
 {
-  enum ImageExtractionMode
-  {
-    ImageExtractionMode_Preview,
-    ImageExtractionMode_UInt8,
-    ImageExtractionMode_UInt16
-  };
-
   enum DicomRootLevel
   {
     DicomRootLevel_Patient,
