@@ -43,6 +43,7 @@ namespace Orthanc
     OFCondition findScp(T_ASC_Association * assoc, 
                         T_DIMSE_Message * msg, 
                         T_ASC_PresentationContextID presID,
-                        IFindRequestHandler& handler);
+                        IFindRequestHandler& handler,
+                        const std::string& callingAETitle);
   }
 }
