@@ -42,11 +42,12 @@ namespace Orthanc
   class OrthancRestApi : public RestApi
   {
   public:
-    typedef std::set<std::string> Modalities;
+    typedef std::set<std::string> SetOfStrings;
 
   private:
     ServerContext& context_;
-    Modalities modalities_;
+    SetOfStrings modalities_;
+    SetOfStrings peers_;
 
   public:
     OrthancRestApi(ServerContext& context);
@@ -56,9 +57,14 @@ namespace Orthanc
       return context_;
     }
 
-    Modalities& GetModalities()
+    SetOfStrings& GetModalities()
     {
       return modalities_;
     }
+
+    SetOfStrings& GetPeers()
+    {
+      return peers_;
+    }
   };
 }
